"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_framework_output_1 = require("@ionic/cli-framework-output");
const chalk = require("chalk");
const os = require("os");
const path = require("path");
const untildify = require("untildify");
var cli_framework_output_2 = require("@ionic/cli-framework-output");
exports.TTY_WIDTH = cli_framework_output_2.TTY_WIDTH;
exports.indent = cli_framework_output_2.indent;
exports.sliceAnsi = cli_framework_output_2.sliceAnsi;
exports.stringWidth = cli_framework_output_2.stringWidth;
exports.stripAnsi = cli_framework_output_2.stripAnsi;
exports.wordWrap = cli_framework_output_2.wordWrap;
function prettyPath(p) {
    p = expandPath(p);
    const cwd = process.cwd();
    const d = path.dirname(p);
    const h = os.homedir();
    const distanceFromCwd = Math.abs(d.split(path.sep).length - cwd.split(path.sep).length);
    if (cwd === d) {
        return '.' + path.sep + path.basename(p);
    }
    else if (d.startsWith(cwd)) {
        return '.' + path.sep + p.substring(cwd.length + 1);
    }
    else if (distanceFromCwd <= 2) {
        const rel = path.relative(cwd, p);
        return rel ? rel : '.';
    }
    else if (p === h) {
        return '~';
    }
    else if (p.indexOf(h) === 0) {
        return '~' + path.sep + p.substring(h.length + 1);
    }
    return p;
}
exports.prettyPath = prettyPath;
function expandPath(p) {
    return path.resolve(untildify(p));
}
exports.expandPath = expandPath;
function generateFillSpaceStringList(list, optimalLength = 1, fillCharacter = ' ') {
    if (optimalLength < 2) {
        optimalLength = 2;
    }
    const longestItem = Math.max(...list.map(item => cli_framework_output_1.stringWidth(item)));
    const fullLength = longestItem > optimalLength ? longestItem + 1 : optimalLength;
    const fullLengthString = fillCharacter.repeat(fullLength);
    return list.map(item => cli_framework_output_1.sliceAnsi(fullLengthString, 0, fullLength - cli_framework_output_1.stringWidth(item)));
}
exports.generateFillSpaceStringList = generateFillSpaceStringList;
/**
 * Basic CLI table generator with support for ANSI colors.
 *
 * @param rows 2-dimensional matrix containing cells. An array of columns,
 *             which are arrays of cells.
 * @param options.vsep The vertical separator character, default is
 *                     `chalk.dim('|')`. Supply an empty string to hide
 *                     the separator altogether.
 * @param options.hsep The horizontal separator character, default is
 *                     `chalk.dim('-')`. This is used under the headers,
 *                     if supplied. Supply an empty string to hide the
 *                     separator altogether.
 * @param options.headers An array of header cells.
 */
function columnar(rows, { hsep = chalk.dim('-'), vsep = chalk.dim('|'), headers }) {
    const includeHeaders = headers ? true : false;
    if (!rows[0]) {
        return '';
    }
    const columnCount = headers ? headers.length : rows[0].length;
    const columns = headers ?
        headers.map(header => [chalk.bold(header)]) :
        rows[0].map(() => []);
    for (const row of rows) {
        let highestLineCount = 0;
        const splitRows = row.map(cell => {
            const lines = cell.split('\n');
            highestLineCount = Math.max(highestLineCount, lines.length);
            return lines;
        });
        for (const rowIndex in row) {
            if (columns[rowIndex]) {
                columns[rowIndex].push(...splitRows[rowIndex], ...Array(highestLineCount - splitRows[rowIndex].length).fill(''));
            }
        }
    }
    const paddedColumns = columns.map((col, columnIndex) => {
        if (columnIndex < columnCount - 1) {
            const spaceCol = generateFillSpaceStringList(col);
            return col.map((cell, cellIndex) => `${cell}${spaceCol[cellIndex]}${vsep === '' ? '' : `${vsep} `}`);
        }
        else {
            return col;
        }
    });
    let longestRowLength = 0;
    const singleColumn = paddedColumns.reduce((a, b) => {
        return a.map((_, i) => {
            const r = a[i] + b[i];
            longestRowLength = Math.max(longestRowLength, cli_framework_output_1.stringWidth(r));
            return r;
        });
    });
    if (includeHeaders && hsep !== '') {
        singleColumn.splice(1, 0, hsep.repeat(longestRowLength));
    }
    return singleColumn.join('\n');
}
exports.columnar = columnar;
